
#ifndef POSTPROC_H
#define POSTPROC_H

/* #define PostCartoonPatch */

/* post-process requirement flags */
#define PP_FLAG_NONE    0x000
#define PP_FLAG_DEPTH   0x001
#define PP_FLAG_COLOUR  0x002
#define PP_FLAG_IPOINT  0x004
#define PP_FLAG_INORMAL 0x008
#define PP_FLAG_PNORMAL 0x010
#define PP_FLAG_IUV     0x020
#define PP_FLAG_OBJECT  0x040

struct post_process_data_lists
{
  DBL **Depth;
  COLOUR **Colour;
  VECTOR **IPoint;
  VECTOR **INormal;
  VECTOR **PNormal;
  UV_VECT **Iuv;
  int **Object;
};

struct post_process_data
{
  DBL Depth;
  COLOUR Colour;
  VECTOR IPoint;
  VECTOR INormal;
  VECTOR PNormal;
  UV_VECT Iuv;
  int Object;
};
typedef struct post_process_data PP_DATA;
typedef struct post_process_data_lists PP_DATA_LISTS;
typedef struct post_process_task PP_TASK;

typedef void (*POST_PROCESS_METHOD)(PP_TASK *, COLOUR **, COLOUR **, PP_DATA_LISTS, int, int);

#define POST_PROCESS_FIELDS PP_TASK *next; \
  POST_PROCESS_METHOD doPostProcess;

struct post_process_task
{
  POST_PROCESS_FIELDS
};

void Assign_PP_Data(PP_DATA *pp_data, INTERSECTION *Inter, COLOUR Colour, DBL depth);
void Write_Post_Line(PP_DATA *Line, int y);
void Read_Post_Line(PP_DATA_LISTS Line, int y, int width);
void Open_Post_File();
void Close_Post_File();
void Delete_Post_File();
void DoPostProcess(char* filename);
void Deinitialize_PostProcessing(void);
void Initialize_PostProcessing(void);
void Open_Post_Input();



/* ================================================== */
/*  Focal Blur post process */
/* ================================================== */

struct post_process_focal_blur
{
  POST_PROCESS_FIELDS
  DBL fieldStart;
  DBL fieldDepth;
  int maxPixelBlur;
};
typedef struct post_process_focal_blur PP_FOCAL_BLUR;
PP_FOCAL_BLUR *createPostFocalBlur(void);
void doPostFocalBlur(PP_TASK *, COLOUR **, COLOUR **, PP_DATA_LISTS, int, int);
#define PP_FOCAL_BLUR_FLAGS PP_FLAG_DEPTH

/* ================================================== */
/*  Depth post process */
/* ================================================== */

struct post_process_depth
{
  POST_PROCESS_FIELDS
  DBL fieldDepth;
  DBL fieldStart;
};
typedef struct post_process_depth PP_DEPTH;
PP_DEPTH *createPostDepth(void);
void doPostDepth(PP_TASK *, COLOUR **, COLOUR **, PP_DATA_LISTS, int, int);
#define PP_DEPTH_FLAGS PP_FLAG_DEPTH

/* ============================================================ */
/*  Soft Glow post process */
/* ============================================================ */

struct post_process_soft_glow
{
  POST_PROCESS_FIELDS
  DBL glowAmount;
  DBL blurAmount;
};
typedef struct post_process_soft_glow PP_SOFT_GLOW;
PP_SOFT_GLOW *createPostSoftGlow(void);
void doPostSoftGlow(PP_TASK *, COLOUR **, COLOUR **, PP_DATA_LISTS, int, int);
#define PP_SOFT_GLOW_FLAGS PP_FLAG_NONE

#ifdef PostCartoonPatch
/* ============================================================ */
/*  Cartoon post process */
/* ============================================================ */

struct post_process_cartoon
{
  POST_PROCESS_FIELDS
  DBL blurAmount;
  DBL hueThreshold;
  DBL lumThreshold;

  int lineWidth;
  COLOUR lineColour;
  DBL lineHueThreshold;
};
typedef struct post_process_cartoon PP_CARTOON;
PP_CARTOON *createPostCartoon(void);
void doPostCartoon(PP_TASK *, COLOUR **, COLOUR **, PP_DATA_LISTS, int, int);
#define PP_CARTOON_FLAGS PP_FLAG_NONE
#endif

#endif
